# TPK4186 - Advanced Tools for Performance Engineering
# ComplexNumbers.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Required modules
import sys
import math

# Creation
def ComplexNew(a, b):
  return (a, b)

def ComplexRealPart(c):
  return c[0]

def ComplexImaginaryPart(c):
  return c[1]

# Operations
def ComplexConjugate(c):
  return ComplexNew(ComplexRealPart(c), -ComplexImaginaryPart(c))

def ComplexAdd(c1, c2):
  a1 = ComplexRealPart(c1)
  b1 = ComplexImaginaryPart(c1)
  a2 = ComplexRealPart(c2)
  b2 = ComplexImaginaryPart(c2)
  return ComplexNew(a1+a2, b1+b2)

def ComplexSub(c1, c2):
  a1 = ComplexRealPart(c1)
  b1 = ComplexImaginaryPart(c1)
  a2 = ComplexRealPart(c2)
  b2 = ComplexImaginaryPart(c2)
  return ComplexNew(a1-a2, b1-b2)

def ComplexMul(c1, c2):
  a1 = ComplexRealPart(c1)
  b1 = ComplexImaginaryPart(c1)
  a2 = ComplexRealPart(c2)
  b2 = ComplexImaginaryPart(c2)
  return ComplexNew(a1*a2-b1*b2, a1*b2+a2*b1)

def ComplexDiv(c1, c2):
  # (a1 + b1.i)/(a2 + b2.i) = (a1 + b1.i)x(a2 - b2.i)/(a2 + b2.i)x(a2 - b2.i)
  #                         = [a1*a2 + b1*b2 + (a2*b1-a1*b1).i]/[a2^2 + b2^2]
  a1 = ComplexRealPart(c1)
  b1 = ComplexImaginaryPart(c1)
  a2 = ComplexRealPart(c2)
  b2 = ComplexImaginaryPart(c2)
  d = a2**2 + b2**2
  return ComplexNew((a1*a2+b1*b2)/d, (a2*b1-a1*b2)/d)

def ComplexPow(c, n):
  if n==0:
    return ComplexNew(1, 0)
  if n==1:
    return c
  k = n//2
  d = ComplexPow(c, k)
  r = ComplexMul(d, d)
  if n%2==0:
    return r
  return ComplexMul(r, c)

# Printing
def ComplexPrint(c):
  print(str(ComplexRealPart(c)) + " + " + str(ComplexImaginaryPart(c)) + ".i")

# Calculations
c1 = ComplexDiv(ComplexNew(-4, 1), ComplexNew(-3, 2))
ComplexPrint(c1)

d21 = ComplexDiv(ComplexNew(1, 1), ComplexNew(1, -1))
d22 = ComplexMul(ComplexNew(1, 2), ComplexNew(2, 2))
d23 = ComplexSub(d21, d22)
d24 = ComplexDiv(ComplexNew(3, -1), ComplexNew(1, 1))
c2 = ComplexAdd(d23, d24)
ComplexPrint(c2)

d31 = ComplexMul(ComplexNew(0, 2), ComplexNew(-1, 1))
d32 = ComplexConjugate(ComplexNew(math.sqrt(3), 1))
d33 = ComplexPow(d32, 3)
d34 = ComplexAdd(d31, d33)
d35 = ComplexNew(1, 1)
d36 = ComplexConjugate(d35)
d37 = ComplexMul(d35, d36)
c3 = ComplexAdd(d34, d37)
ComplexPrint(c3)



