# TPK4186 - Advanced Tools for Performance Engineering
# ReliabilityKOutOfNSystems.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.


# Required modules
import sys
import math
import random

# Drawing lists of probabilities
def DrawListOfProbability(size, lowerBound, upperBound):
  return [random.uniform(lowerBound, upperBound) for i in range(0, size)]

# Caclulation of the probability
def CalculateProbability(probabilities, n, k):
  if k<=0:
    return 1.0
  if k>n:
    return 0.0
  p = probabilities[n-1]
  return p*CalculateProbability(probabilities, n-1, k-1) + (1-p)*CalculateProbability(probabilities, n-1, k)

# Caclulation of the probability with cache
def CalculateProbabilityWithCache(probabilities, n, k, cache):
  if k<=0:
    return 1.0
  if k>n:
    return 0.0
  r = cache.get((n, k), None)
  if r!=None:
    return r
  p = probabilities[n-1]
  r = p*CalculateProbabilityWithCache(probabilities, n-1, k-1, cache) + (1-p)*CalculateProbabilityWithCache(probabilities, n-1, k, cache)
  cache[(n, k)] = r
  return r

# Tests
random.seed(12345)

P1 = DrawListOfProbability(20, 0.1, 0.2)
p = CalculateProbability(P1, 20, 10)
print("p1 = " + str(p))

P2 = DrawListOfProbability(24, 0.1, 0.2)
p = CalculateProbability(P2, 24, 10)
print("p2 = " + str(p))

P3 = DrawListOfProbability(28, 0.1, 0.2)
p = CalculateProbability(P3, 28, 10)
print("p3 = " + str(p))


cache = dict()
p = CalculateProbabilityWithCache(P1, 20, 10, cache)
print("p1 = " + str(p))

cache.clear()
p = CalculateProbabilityWithCache(P2, 24, 10, cache)
print("p2 = " + str(p))

cache.clear()
p = CalculateProbabilityWithCache(P3, 28, 10, cache)
print("p3 = " + str(p))

