# TPK4186 - Advanced Tools for Performance Engineering
# TrainRegularity.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import sys

def CalculateMean(values):
  if len(values)==0:
    return 0.0
  sum = Sum(values)
  return sum/len(values)

def AddValueToDictionary(dictionary, key, value):
  values = dictionary.get(key, [])
  if values==[]:
    dictionary[key] = values
  values.append(value)
  

def ImportFile(fileName, trains):
  try:
    file = open(fileName, "r")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  header = True
  for line in file:
    if header:
      header = False
    else:
      line = line.rstrip()
      tokens = line.split(",")
      category = tokens[1]
      line = tokens[2]
      if tokens[3]!="":
        regularity = float(tokens[3])
        key = (category, line)
        AddValueToDictionary(trains, key, regularity)

def ExportRegularities(trains, fileName):
  try:
    file = open(fileName, "w")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  for key in trains:
    regularities = trains[key]
    (category, line) = key
    file.write(category + "\t" + line + "\t")
    first = True
    for regularity in regularities:
      if first:
        first = False
      else:
        file.write(", ")
      file.write(str(regularity))
    file.write("\n")

def ExportMeanRegularities(trains, fileName):
  try:
    file = open(fileName, "w")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  for key in trains:
    regularities = trains[key]
    (category, line) = key
    regularity = CalculateMean(regularities)
    file.write(category + "\t" + line + "\t" + "{:2.2f}".format(regularity) + "\n")
  

trains = dict()
ImportFile("regularite.csv", trains)
ExportRegularities(trains, "regulaties.tsv")
ExportMeanRegularities(trains, "mean-regulaties.tsv")
